#include "handmade.h"

#define PI32 3.141592653589f

void debug_printf(wchar_t* format, ...) { 
    int bufsize = wcslen(format)*10;
    wchar_t *output = (wchar_t*)malloc(bufsize);
    va_list list;
    va_start(list, format);
    vswprintf(output, bufsize, format, list);
    OutputDebugStringW(output);
    free(output);
}

internal void outputSound(GameSoundOutputBuffer *soundBuffer) {
    local_persist real32 tSine;
    int16 toneVolume = 3000;
    int toneHz = 440;
    int wavePeriod = soundBuffer->samplesPerSecond/toneHz;

    int16 *sampleOut = soundBuffer->samples;
    for (int sampleIndex = 0; sampleIndex < soundBuffer->sampleCount; sampleIndex++) {
        int16 sampleValue = (int16)(sin(tSine) * (real32)toneVolume);
        *sampleOut++ = sampleValue;
        *sampleOut++ = sampleValue;
        tSine += 2.0f * PI32 / (real32)wavePeriod;
    }
}

internal void renderWeirdGradient(GameOffscreenBuffer *buffer, int xOffset, int yOffset) {
    int bytesPerPixel = 4;
    int pitch = buffer->width*bytesPerPixel;
    uint8 *row = (uint8 *)buffer->memory;
    for (int y = 0; y < buffer->height; y++) {
        uint32 *pixel = (uint32*)row;
        for (int x = 0; x < buffer->width; x++) {
            uint8 blue = x + xOffset;
            uint8 green = y + yOffset;
            *pixel++ = (green << 8) | blue;
        }
        row += pitch;
    }
}

internal void gameUpdateAndRender(GameOffscreenBuffer *videoBuf, GameInput *input, GameSoundOutputBuffer *soundBuf) {
    outputSound(soundBuf);
    renderWeirdGradient(videoBuf, input->xOffset, input->yOffset);
}
